/**
* Ein HTML-Editor mit Ergebnisvorschau
*/
function HTMLEditor(
    parentNode      // Elternelement, in dem sich der Editor befinden soll
) {
    this.establish(parentNode);
}

// Funktion, die bei einer Änderung des HTML-Quelltextes ausgeführt werden soll
HTMLEditor.prototype.onchange = undefined;

/**
* HTML-Element in DOM-Baum einfügen
* @return Element   Erzeugter Knoten
*/
HTMLEditor.createHtml = function(
    strTagname,     // Optional: Tagname des zu erzeugenden Elementes
    nodeParent,     // Optional: Künftiger Elternknoten
    objAtts,        // Optional: Attribute des zu erzeugenden Elementes
    strTextContent, // Optional: Textinhalt des Elementes
    nodeBefore      // Optional: Knoten, als deren previousSibling der neue Knoten eingefügt werden soll
) {
    var node = nodeBefore ? nodeBefore.parentNode : nodeParent;
    if (nodeBefore && nodeBefore.parentNode)
        nodeParent = nodeBefore.parentNode;
    if (strTagname)
        node = (nodeParent ? nodeParent.ownerDocument : document).createElementNS('htt' + 'p://w' + 'ww.w3.org/1999/xhtml', strTagname);
    if (objAtts)
        for (var key in objAtts)
            node.setAttribute(key, objAtts[key]);
    if ((strTextContent !== undefined) && ('' + strTextContent).length)
        node.appendChild((nodeParent ? nodeParent.ownerDocument : document).createTextNode(strTextContent));
    if (nodeParent && (node != nodeParent))
        nodeParent.insertBefore(node, nodeBefore);
    return node;
}

/**
* Editor-DOM erstellen
* @return undefined
*/
HTMLEditor.prototype.establish = function(
    parentNode      // Elternelement, in dem sich der Editor befinden soll
) {
    HTMLEditor.createHtml('h2', parentNode, null, 'Live-Vorschau');
    this.nodeIframe = HTMLEditor.createHtml('iframe', HTMLEditor.createHtml('div', parentNode, { 'class': 'resizable' }), { src: 'about:blank' });
    HTMLEditor.createHtml('h2', parentNode, null, 'HTML-Quelltext');
    this.editor = CodeMirror.fromTextArea(
        HTMLEditor.createHtml('textarea', parentNode), {
        mode: 'text/html',
        indentUnit: 4,
        lineWrapping: true,
        lineNumbers: true,
        matchTags: {bothTags: true},
        styleActiveLine: true,
        viewportMargin: Infinity
    });
    var self = this;
    var delay;
    this.editor.on("change", function() {
        clearTimeout(delay);
        delay = setTimeout(function() { self.updatePreview(); }, 300);
    });
}

/**
* Von Element und allen Kindelementen Attribute entfernen, die mit 'on' beginnen
* @return undefined
*/
HTMLEditor.stripOn = function(
    element     // Element, das rekursive bearbeitet werden soll
) {
    var z, atts;
    if (element.hasAttributes()) {
        var arrKill = [];
        atts = element.attributes
        for (z = 0; z < atts.length; ++z) {
            if (atts[z].name.substr(0, 2).toLowerCase() == 'on')
                arrKill.push(atts[z].name);
        }
        arrKill.map(function(att) { element.removeAttribute(att); });
    }
    for (z = element.firstChild; z; z = z.nextSibling) {
        if (z.nodeType == Node.ELEMENT_NODE)
            HTMLEditor.stripOn(z);
    }
}

/**
* HTML-Eingabe ggf. durch Fehlermeldung ersetzen bzw. anpassen
* @return String    Überarbeitetes HTML
*/
HTMLEditor.prototype.validateHTML = function(
    html        // Editoreingabe: HTML
) {
    var parser = new DOMParser();
    var doc = parser.parseFromString(html, "text/html");
    if (doc.getElementsByTagName('script').length || doc.getElementsByTagName('iframe').length)
        return '<h1>Sicherheitsfehler</h1><p>Die Verwendung von <code>&lt;script&gt;</code>, <code>&lt;iframe&gt;</code> und JavaScript ist aus Sicherheitsgründen nicht erlaubt…</p>';
    // Links sollen in neuem Fenster öffnen
    var nodes = doc.getElementsByTagName('a');
    var z;
    for (z = 0; z < nodes.length; ++z) {
        if (nodes[z].hasAttribute('href'))
            nodes[z].setAttribute('target', '_blank');
    }
    // on*-Attribute entfernen
    HTMLEditor.stripOn(doc.documentElement);
    return (new XMLSerializer()).serializeToString(doc);
}

/**
* Eingegebenes HTML anzeigen
* @return undefined
*/
HTMLEditor.prototype.updatePreview = function() {
    var preview = this.nodeIframe.contentDocument ||  this.nodeIframe.contentWindow.document;
    preview.open();
    preview.write(this.validateHTML(this.editor.getValue()));
    preview.close();
    if (typeof this.onchange == 'function')
        this.onchange.call(this, this.editor.getValue());
}

// Eingegebenes HTML
Object.defineProperty(HTMLEditor.prototype, 'value', {
    enumerable: true,
    get() { return this.editor.getValue(); },
    set(v) {
        this.editor.setValue(v);
        this.updatePreview();
    }
});
